{******************************************************************************}
{**                                                                          **}
{**  TantDatePanel Component. Usefull Panel for input-output Date values.    **}
{**  For Delphi 1-5, any version of C++ Builder not supported.               **}
{**                                                                          **}
{**  Author:    KARPOLAN                                                     **}
{**  E-Mail:    karpolan@yahoo.com , karpolan@utilmind.com                   **}
{**  Home Page: http://karpolan.i.am, http://www.utilmind.com                **}
{**  Copyright  1996-99 by KARPOLAN.                                        **}
{**  Copyright  1999, UtilMind Solutions.                                   **}
{**                                                                          **}
{******************************************************************************}
{** History:
  11 jan 1997 - 1.0 First release....
  ...
  31 aug 1999 - 2.0 Prepared for public release. Some changes...
*******************************************************************************}
Unit antDatePanel;

{$Include antPanels.inc}

Interface

{$IfDef Delphi - NOTHING FOR BUILDER}

Uses
  Spin,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls;

Type
  EantDatePanel = Class(Exception);

{##############################################################################}
{## TantDatePanel = Class(TCustomPanel) #######################################}
{##############################################################################}
{*******************************************************************************
   Usefull Panel for input-output Date values. Full bug for 30th of February
and 31th of April, Right Leap Years, Y2K routines.
Methods:
  SetDate - Set date value to Panel (TDateTime),
  GetDate - Get date value from Panel (TDateTime),
  GetDateAsShortStr - Get date value from Panel as String formated like
                      ShortDateFormat.
  GetDateAsLongStr  - Get date value from Panel as String, formated like
                      'DD MMMMMM YYYY'.
Properies:
  Monthes : TString - List of names for all 12 monthes.
Events :
  OnDayChange, OnMonthChange, OnYearChange.

For more information - see Demo project..
*******************************************************************************}
  TantDatePanel = Class(TCustomPanel)
  Private
  {** Events **}
    fOnChangeDay    : TNotifyEvent;
    fOnChangeMonth  : TNotifyEvent;
    fOnChangeYear   : TNotifyEvent;
    fOnDayKeyPress  : TKeyPressEvent;
    fOnYearKeyPress : TKeyPressEvent;
  {** Messages Routine **}
    Procedure WMPaint(Var Message : TWMPaint);                 Message WM_Paint;

  Protected
  {** Components **}
    FcmbMonth  : TComboBox;
    FsedDay,
    FsedYear   : TSpinEdit;
  {** UpDate Routines **}
    Procedure UpDateWidth;                                              Virtual;
  {** Events Routines **}
    Procedure DoChangeDay  (Sender : TObject);                          Virtual;
    Procedure DoChangeMonth(Sender : TObject);                          Virtual;
    Procedure DoChangeYear (Sender : TObject);                          Virtual;
    Procedure DoKeyPress(Sender  : TObject;
                         Var Key : Char);                               Virtual;
  {** Properties Routines **}
    Function  GetItems : TStrings;                                      Virtual;
    Procedure SetItems(A : TStrings);                                   Virtual;
    Function  GetDay      : integer;                                    Virtual;
    Function  GetMonth    : integer;                                    Virtual;
    Function  GetMonthStr : String;                                     Virtual;
    Function  GetYear     : integer;                                    Virtual;
  Public
  {** Init/Done **}
    Constructor Create(AOwner : TComponent);                           Override;
  {** Date Routines **}
    Function  GetDate           : TDateTime;                            Virtual;
    Function  GetDateAsShortStr : String;                               Virtual;
    Function  GetDateAsLongStr  : String;                               Virtual;
    Procedure SetDate(ADate : TDateTime);                               Virtual;
  {** Properties Routine **}
    Property Day        : integer
      Read   GetDay;
    Property Month      : integer
      Read   GetMonth;
    Property MonthStr   : String
      Read   GetMonthStr;
    Property Year       : integer
      Read   GetYear;
    Property sedDay     : TSpinEdit
      Read   FsedDay;
    Property cmbMonth   : TComboBox
      Read   FcmbMonth;
    Property sedYear    : TSpinEdit
      Read   FsedYear;

  Published
  {** New Properties **}
    Property Monthes    : TStrings
      Read   GetItems
      Write  SetItems;
  {** Old Properties **}
    Property Visible;
    Property fOnt;
    Property ParentfOnt;
    Property ShowHint;
    Property ParentShowHint;
  {** Events Properties **}
    Property OnChangeDay     : TNotifyEvent
      Read   fOnChangeDay
      Write  fOnChangeDay;
    Property OnChangeMonth   : TNotifyEvent
      Read   fOnChangeMonth
      Write  fOnChangeMonth;
    Property OnChangeYear    : TNotifyEvent
      Read   fOnChangeYear
      Write  fOnChangeYear;
    Property OnKeyPressDay   : TKeyPressEvent
      Read   fOnDayKeyPress
      Write  fOnDayKeyPress;
    Property OnKeyPressYear  : TKeyPressEvent
      Read   fOnYearKeyPress
      Write  fOnYearKeyPress;
  End;{TantDatePanel = Class(TPanel)}

{$IfDef Delphi2andLower}
Function IsLeapYear(Year: Word): Boolean;
{$EndIf Delphi2andLower}

{$EndIf Delphi - NOTHING FOR BUILDER}

{##############################################################################}
{******************************************************************************}
{##############################################################################}
Implementation
{##############################################################################}
{******************************************************************************}
{##############################################################################}

{$IfDef Delphi - NOTHING FOR BUILDER}

{$IfDef Delphi2andLower}
Function IsLeapYear(Year: Word): Boolean;
Begin
  Result := (Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0));
End;
{$EndIf Delphi2andLower}

{##############################################################################}
{## TantDatePanel = Class(TCustomPanel) #######################################}
{##############################################################################}
{******************************************************************************}
{** Public ********************************************************************}
{******************************************************************************}
{** Init/Done **}
{***************}
Constructor TantDatePanel.Create(AOwner : TComponent);
Begin
  Inherited Create(AOwner);
  Caption     := ' ';
  BevelOuter  := bvNone;
  BevelInner  := bvNone;
  Ctl3D       := True;
  ParentCtl3D := False;

  FsedDay := TSpinEdit.Create(Self);
    FsedDay.Parent     := Self;
    FsedDay.MinValue   := 1;
    FsedDay.MaxValue   := 31;
    FsedDay.MaxLength  := 2;
    FsedDay.Value      := 1;
    FsedDay.OnChange   := DoChangeDay;   {}
    FsedDay.OnKeyPress := DoKeyPress; {}
    FsedDay.Left       := 1;
    FsedDay.Top        := 1;
    FsedDay.Show;

  FcmbMonth := TComboBox.Create(Self);
    FcmbMonth.Parent        := Self;
    FcmbMonth.Style         := csDropDownList;  {}
    FcmbMonth.DropDownCount := 12;
    FcmbMonth.OnChange      := DoChangeMonth;   {}
    FcmbMonth.Top           := 2;
    FcmbMonth.Show;

  FsedYear := TSpinEdit.Create(Self);
    FsedYear.Parent     := Self;
    FsedYear.MinValue   := 0;
    FsedYear.MaxValue   := 9999;
    FsedYear.MaxLength  := 4;
    FsedYear.Value      := 1980;
    FsedYear.OnChange   := DoChangeYear;   {}
    FsedYear.OnKeyPress := DoKeyPress; {}
    FsedYear.Top        := 1;
    FsedYear.Show;
End;{Constructor TantDatePanel.Create}

{*******************}
{** Date Routines **}
{*******************}
Function TantDatePanel.GetDate : TDateTime;
Var
  RegStr : String;
Begin
  Result := 0;
  RegStr := ShortDateFormat;
  try
    ShortDateFormat := 'D' + DateSeparator+
                       'M' + DateSeparator+
                       'Y';
    Result := StrToDate(IntToStr(Day)   + DateSeparator +
                        IntToStr(Month) + DateSeparator +
                        IntToStr(Year));
  except
//    Raise EantDatePanel.CreateFmt('%s.GetDate failed', [Name]);
  end; {try .. except}
  ShortDateFormat := RegStr;
End;{Function TantDatePanel.GetDateAsDateTime}

Function TantDatePanel.GetDateAsShortStr : String;
Begin
  Result:=DateToStr(GetDate);
End;{Function TantDatePanel.GetDateAsShortStr}

Function TantDatePanel.GetDateAsLongStr  : String;
Begin
  Result := IntToStr(Day) + ' ' +
            MonthStr      + ' ' +
            IntToStr(Year);
End;{Function TantDatePanel.GetDateAsLongStr}

Procedure TantDatePanel.SetDate(ADate : TDateTime);
Var
  RegStr : String;
Begin
  RegStr := FormatDateTime('DDMMYYYY',ADate);
  sedDay. Text := Copy(RegStr, 1, 2);
  sedYear.Text := Copy(RegStr, 5, 4);
  try
    cmbMonth.ItemIndex := StrToInt(Copy(RegStr,3,2))-1;
  except
  end;{try .. except}
  DoChangeYear (fsedYear);
  DoChangeMonth(fcmbMonth);
End;{Procedure TantDatePanel.SetDate}

{******************************************************************************}
{** Protected *****************************************************************}
{******************************************************************************}
{** UpDate Routines **}
{*********************}
Procedure TantDatePanel.UpDateWidth;
Begin
  Canvas.fOnt     := FsedDay.Font;
  FsedDay.  Width := Canvas.TextWidth('00') + 24;

  FcmbMonth.Left  := FsedDay.BoundsRect.Right + 4;
  FcmbMonth.Width := Canvas.TextWidth('XXXXXXXXXX') + 24;

  FsedYear. Left  := FcmbMonth.BoundsRect.Right + 4;
  FsedYear. Width := Canvas.TextWidth('0000') + 24;

  Width  := FsedYear.BoundsRect.Right + 1;
  Height := FsedDay.Height + 2;
End;{Procedure TantDatePanel.UpDateWidth;}

{*********************}
{** Events Routines **}
{*********************}
Procedure TantDatePanel.DoChangeDay(Sender : TObject);
Begin
  if Assigned(fOnChangeDay)
   then fOnChangeDay(Sender);
  with TSpinEdit(Sender) do
   begin
     if (Value < MinValue) then
      begin
        Value := MinValue;
        Exit;
      end;{if Value < MinValue then}
     if (Value > 30) and (Month in [4, 6, 9, 11]) then
      begin
        Value := 30;
        Exit;
      end;{if (Value > 30) and (Month in [4,6,9,11]) then}
     if (Month = 2) then        {** February **}
      begin
        if IsLeapYear(Year) then
         begin                  {** 366 Day In The Year **}
           if Value > 29
            then Value := 29;
         end else
         begin                  {** 365 Day In The Year **}
           if Value > 28
            then Value := 28;
         end;{if ((Year mod 4) = 0) else}
      end;{if (Month = 2) then}
   end;{with TSpinEdit(Sender) do}
End;{Procedure TantDatePanel.DoChangeDay}

Procedure TantDatePanel.DoChangeMonth(Sender : TObject);
Begin
  if Assigned(fOnChangeMonth)
   then fOnChangeMonth(Sender);
  FsedDay.OnChange(FsedDay); {** Check The Day **}
End;{Procedure TantDatePanel.DoChangeMonth}

Procedure TantDatePanel.DoChangeYear(Sender : TObject);
Begin
  if Assigned(fOnChangeYear)
   then fOnChangeYear(Sender);
  FsedDay.OnChange(FsedDay); {** Check The Day **}
End;{Procedure TantDatePanel.DoChangeYear}

Procedure TantDatePanel.DoKeyPress(Sender  : TObject;
                                Var Key : Char);
Begin
  if Assigned(fOnDayKeyPress)
   then fOnDayKeyPress(Sender,Key);
  if not (Key in [#1..#31,'0'..'9']) then
   begin
     MessageBeep(0);
     Key := #0;
   end;{if not (Key in [#1..#31,'0'..'9']) then}
End;{Procedure TantDatePanel.DoKeyPress}

{*************************}
{** Properties Routines **}
{*************************}
Function TantDatePanel.GetItems : TStrings;
Begin
  if FcmbMonth.Items <> nil
   then Result := FcmbMonth.Items
   else Result := nil;{}
End;
Procedure TantDatePanel.SetItems(A : TStrings);
Begin
  FcmbMonth.Items.Assign(A);
End;

Function TantDatePanel.GetDay   : integer;
Begin
  Result := FsedDay.Value;
  if Result < 1
   then Result := 1;
End;

Function TantDatePanel.GetMonth : integer;
Begin
  Result := FcmbMonth.ItemIndex + 1;
  if Result < 1
   then Result := 1;
End;

Function TantDatePanel.GetMonthStr : String;
Begin
  Result := FcmbMonth.Text;
End;

Function TantDatePanel.GetYear  : integer;
Begin
  Result:=FsedYear.Value;
End;

{******************************************************************************}
{** Private *******************************************************************}
{******************************************************************************}
{** Message Routine **}
{*********************}
Procedure TantDatePanel.WMPaint(Var Message : TWMPaint);
Begin
  UpDateWidth;
  Inherited;
End;{Procedure TantDatePanel.WMPaint}


{##############################################################################}
{******************************************************************************}
{##############################################################################}
Initialization
  RegisterClasses([TComboBox, TSpinEdit]);

{$EndIf Delphi - NOTHING FOR BUILDER}

END{Unit antDatePanel}.

